@@ -2,14 +2,16 @@
 
 import os
 import unittest
+from typing import Any
 from unittest import mock
 
 from parameterized import parameterized
 
+from podman_compose import PodmanCompose
 from podman_compose import container_to_args
 
 
-def create_compose_mock(project_name="test_project_name"):
+def create_compose_mock(project_name: str = "test_project_name") -> PodmanCompose:
     compose = mock.Mock()
     compose.project_name = project_name
     compose.dirname = "test_dirname"
@@ -18,15 +20,17 @@ def create_compose_mock(project_name="test_project_name"):
     compose.default_net = None
     compose.networks = {}
     compose.x_podman = {}
+    compose.join_name_parts = mock.Mock(side_effect=lambda *args: '_'.join(args))
+    compose.format_name = mock.Mock(side_effect=lambda *args: '_'.join([project_name, *args]))
 
-    async def podman_output(*args, **kwargs):
+    async def podman_output(*args: Any, **kwargs: Any) -> None:
         pass
 
     compose.podman.output = mock.Mock(side_effect=podman_output)
     return compose
 
 
-def get_minimal_container():
+def get_minimal_container() -> dict[str, Any]:
     return {
         "name": "project_name_service_name1",
         "service_name": "service_name",
@@ -34,13 +38,13 @@ def get_minimal_container():
     }
 
 
-def get_test_file_path(rel_path):
+def get_test_file_path(rel_path: str) -> str:
     repo_root = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
     return os.path.realpath(os.path.join(repo_root, rel_path))
 
 
 class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
-    async def test_minimal(self):
+    async def test_minimal(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -56,7 +60,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_runtime(self):
+    async def test_runtime(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -75,7 +79,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_sysctl_list(self):
+    async def test_sysctl_list(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -99,7 +103,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_sysctl_map(self):
+    async def test_sysctl_map(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -123,7 +127,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_sysctl_wrong_type(self):
+    async def test_sysctl_wrong_type(self) -> None:
         c = create_compose_mock()
         cnt = get_minimal_container()
 
@@ -133,7 +137,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
                 cnt["sysctls"] = wrong_type
                 await container_to_args(c, cnt)
 
-    async def test_pid(self):
+    async def test_pid(self) -> None:
         c = create_compose_mock()
         cnt = get_minimal_container()
 
@@ -152,7 +156,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_http_proxy(self):
+    async def test_http_proxy(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -170,7 +174,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_uidmaps_extension_old_path(self):
+    async def test_uidmaps_extension_old_path(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -179,7 +183,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
         with self.assertRaises(ValueError):
             await container_to_args(c, cnt)
 
-    async def test_uidmaps_extension(self):
+    async def test_uidmaps_extension(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -200,7 +204,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_gidmaps_extension(self):
+    async def test_gidmaps_extension(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -221,7 +225,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_rootfs_extension(self):
+    async def test_rootfs_extension(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -240,7 +244,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_no_hosts_extension(self):
+    async def test_no_hosts_extension(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -258,11 +262,47 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_env_file_str(self):
+    @parameterized.expand([
+        # short syntax: only take this specific environment variable value from .env file
+        ("use_env_var_from_default_env_file_short_syntax", ["ZZVAR1"], "ZZVAR1=TEST1"),
+        # long syntax: environment variable value from .env file is taken through variable
+        # interpolation
+        # only the value required in 'environment:' compose file is sent to containers
+        # environment
+        ("use_env_var_from_default_env_file_long_syntax", ["ZZVAR1=TEST1"], "ZZVAR1=TEST1"),
+        # "environment:" section in compose file overrides environment variable value from .env file
+        (
+            "use_env_var_from_default_env_file_override_value",
+            ["ZZVAR1=NEW_TEST1"],
+            "ZZVAR1=NEW_TEST1",
+        ),
+    ])
+    async def test_env_file(self, test_name: str, cnt_env: list, expected_var: str) -> None:
+        c = create_compose_mock()
+        # environment variables were set in .env file
+        c.environ = {"ZZVAR1": "TEST1", "ZZVAR2": "TEST2"}
+
+        cnt = get_minimal_container()
+        cnt["environment"] = cnt_env
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "-e",
+                f"{expected_var}",
+                "--network=bridge:alias=service_name",
+                "busybox",
+            ],
+        )
+
+    async def test_env_file_str(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
-        env_file = get_test_file_path('tests/integration/env-file-tests/env-files/project-1.env')
+        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
         cnt['env_file'] = env_file
 
         args = await container_to_args(c, cnt)
@@ -282,7 +322,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_env_file_str_not_exists(self):
+    async def test_env_file_str_not_exists(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -291,11 +331,11 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
         with self.assertRaises(ValueError):
             await container_to_args(c, cnt)
 
-    async def test_env_file_str_array_one_path(self):
+    async def test_env_file_str_array_one_path(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
-        env_file = get_test_file_path('tests/integration/env-file-tests/env-files/project-1.env')
+        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
         cnt['env_file'] = [env_file]
 
         args = await container_to_args(c, cnt)
@@ -315,12 +355,12 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_env_file_str_array_two_paths(self):
+    async def test_env_file_str_array_two_paths(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
-        env_file = get_test_file_path('tests/integration/env-file-tests/env-files/project-1.env')
-        env_file_2 = get_test_file_path('tests/integration/env-file-tests/env-files/project-2.env')
+        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
+        env_file_2 = get_test_file_path('tests/integration/env_file_tests/env-files/project-2.env')
         cnt['env_file'] = [env_file, env_file_2]
 
         args = await container_to_args(c, cnt)
@@ -344,11 +384,11 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_env_file_obj_required(self):
+    async def test_env_file_obj_required(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
-        env_file = get_test_file_path('tests/integration/env-file-tests/env-files/project-1.env')
+        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
         cnt['env_file'] = {'path': env_file, 'required': True}
 
         args = await container_to_args(c, cnt)
@@ -368,7 +408,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_env_file_obj_required_non_existent_path(self):
+    async def test_env_file_obj_required_non_existent_path(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -377,7 +417,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
         with self.assertRaises(ValueError):
             await container_to_args(c, cnt)
 
-    async def test_env_file_obj_optional(self):
+    async def test_env_file_obj_optional(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -394,7 +434,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_gpu_count_all(self):
+    async def test_gpu_count_all(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -422,7 +462,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_gpu_count_specific(self):
+    async def test_gpu_count_specific(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -458,7 +498,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_gpu_device_ids_all(self):
+    async def test_gpu_device_ids_all(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -492,7 +532,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_gpu_device_ids_specific(self):
+    async def test_gpu_device_ids_specific(self) -> None:
         c = create_compose_mock()
 
         cnt = get_minimal_container()
@@ -529,12 +569,28 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
         )
 
     @parameterized.expand([
-        (False, "z", ["--mount", "type=bind,source=./foo,destination=/mnt,z"]),
-        (False, "Z", ["--mount", "type=bind,source=./foo,destination=/mnt,Z"]),
-        (True, "z", ["-v", "./foo:/mnt:z"]),
-        (True, "Z", ["-v", "./foo:/mnt:Z"]),
+        (
+            False,
+            "z",
+            [
+                "--mount",
+                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt,z",
+            ],
+        ),
+        (
+            False,
+            "Z",
+            [
+                "--mount",
+                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt,Z",
+            ],
+        ),
+        (True, "z", ["-v", f"{get_test_file_path('test_dirname/foo')}:/mnt:z"]),
+        (True, "Z", ["-v", f"{get_test_file_path('test_dirname/foo')}:/mnt:Z"]),
     ])
-    async def test_selinux_volume(self, prefer_volume, selinux_type, expected_additional_args):
+    async def test_selinux_volume(
+        self, prefer_volume: bool, selinux_type: str, expected_additional_args: list
+    ) -> None:
         c = create_compose_mock()
         c.prefer_volume_over_mount = prefer_volume
 
@@ -567,14 +623,72 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
+    @parameterized.expand([
+        (
+            "absolute_path",
+            get_test_file_path('test_dirname/foo'),
+            [
+                "--mount",
+                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt",
+            ],
+        ),
+        (
+            "relative_path",
+            './foo',
+            [
+                "--mount",
+                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt",
+            ],
+        ),
+        (
+            "home_dir",
+            '~/test_dirname/foo',
+            [
+                "--mount",
+                f"type=bind,source={os.path.expanduser('~/test_dirname/foo')},destination=/mnt",
+            ],
+        ),
+    ])
+    async def test_volumes_bind_mount_source(
+        self, test_name: str, mount_source: str, expected_additional_args: list
+    ) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+
+        # This is supposed to happen during `_parse_compose_file`
+        # but that is probably getting skipped during testing
+        cnt["_service"] = cnt["service_name"]
+
+        cnt["volumes"] = [
+            {
+                "type": "bind",
+                "source": f"{mount_source}",
+                "target": "/mnt",
+            }
+        ]
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                *expected_additional_args,
+                "--network=bridge:alias=service_name",
+                "busybox",
+            ],
+        )
+
     @parameterized.expand([
         ("not_compat", False, "test_project_name", "test_project_name_network1"),
         ("compat_no_dash", True, "test_project_name", "test_project_name_network1"),
         ("compat_dash", True, "test_project-name", "test_projectname_network1"),
     ])
-    async def test_network_default_name(self, name, is_compat, project_name, expected_network_name):
+    async def test_network_default_name(
+        self, name: str, is_compat: bool, project_name: str, expected_network_name: str
+    ) -> None:
         c = create_compose_mock(project_name)
-        c.x_podman = {"default_net_name_compat": is_compat}
+        c.x_podman = {PodmanCompose.XPodmanSettingKey.DEFAULT_NET_NAME_COMPAT: is_compat}
         c.networks = {'network1': {}}
 
         cnt = get_minimal_container()
@@ -591,7 +705,7 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
             ],
         )
 
-    async def test_device(self):
+    async def test_device(self) -> None:
         c = create_compose_mock()
         cnt = get_minimal_container()
 
@@ -612,3 +726,155 @@ class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
                 "busybox",
             ],
         )
+
+    async def test_cpuset(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["cpuset"] = "0-1"
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "--network=bridge:alias=service_name",
+                "--cpuset-cpus",
+                "0-1",
+                "busybox",
+            ],
+        )
+
+    async def test_pids_limit_container_level(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["pids_limit"] = 100
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "--network=bridge:alias=service_name",
+                "--pids-limit",
+                "100",
+                "busybox",
+            ],
+        )
+
+    async def test_pids_limit_deploy_section(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["deploy"] = {"resources": {"limits": {"pids": 100}}}
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "--network=bridge:alias=service_name",
+                "--pids-limit",
+                "100",
+                "busybox",
+            ],
+        )
+
+    async def test_pids_limit_both_same(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["pids_limit"] = 100
+        cnt["deploy"] = {"resources": {"limits": {"pids": 100}}}
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "--network=bridge:alias=service_name",
+                "--pids-limit",
+                "100",
+                "busybox",
+            ],
+        )
+
+    async def test_pids_limit_both_different(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["pids_limit"] = 100
+        cnt["deploy"] = {"resources": {"limits": {"pids": 200}}}
+
+        with self.assertRaises(ValueError):
+            await container_to_args(c, cnt)
+
+    async def test_healthcheck_string(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["healthcheck"] = {
+            "test": "cmd arg1 arg2",
+        }
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "--network=bridge:alias=service_name",
+                "--healthcheck-command",
+                '["CMD-SHELL", "cmd arg1 arg2"]',
+                "busybox",
+            ],
+        )
+
+    async def test_healthcheck_cmd_args(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["healthcheck"] = {
+            "test": ["CMD", "cmd", "arg1", "arg2"],
+        }
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "--network=bridge:alias=service_name",
+                "--healthcheck-command",
+                '["cmd", "arg1", "arg2"]',
+                "busybox",
+            ],
+        )
+
+    async def test_healthcheck_cmd_shell(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["healthcheck"] = {
+            "test": ["CMD-SHELL", "cmd arg1 arg2"],
+        }
+
+        args = await container_to_args(c, cnt)
+        self.assertEqual(
+            args,
+            [
+                "--name=project_name_service_name1",
+                "-d",
+                "--network=bridge:alias=service_name",
+                "--healthcheck-command",
+                '["cmd arg1 arg2"]',
+                "busybox",
+            ],
+        )
+
+    async def test_healthcheck_cmd_shell_error(self) -> None:
+        c = create_compose_mock()
+        cnt = get_minimal_container()
+        cnt["healthcheck"] = {
+            "test": ["CMD-SHELL", "cmd arg1", "arg2"],
+        }
+
+        with self.assertRaises(ValueError):
+            await container_to_args(c, cnt)
